/*====================================================================
Project:    What Messi_ages Encouri_age COVID-19 Vaccination Intentions?: 
			Meta-Analytic Experimental Evidence with Individual 
			Participant Data from 23 Countries
Author:        
----------------------------------------------------------------------
Creation Date:    13 Nov 2022 - 09:58:31
====================================================================*/
clear all
macro drop _all
set matsize 11000, permanently
set more off, perm

global dir "add your path"

* Graph Scheme
set scheme s1color

sysdir set PLUS "$dir/ado"

* Install commands

*ssc install outreg2
*ssc install metan, replace
*ssc install stripplot, replace
*ssc install ipdmetan, replace
*ssc install rsort

/*====================================================================
======================================================================
							Combining Data					
======================================================================
====================================================================*/


use "${dir}\Datasets\belize_raw", replace

foreach country in "chad" "cmr" "congo" "DJ" "haiti" "honduras" "iraq_w1" "iraq_w2" "jamaica" "jordan_w1" "jordan_w2" "ksa" "kuwait" "lebanon_w1" "lebanon_w2" "libya" "MK" "png" "serbia" "ukraine" "wbg" "XK" "zambia" "gambia" "ukraine_sm" "tunisia_w1" "tunisia_w2" "tunisia_w3" {

	di "`country'"
	append using "${dir}\Datasets/`country'_raw.dta"
	
}

save  "${dir}\Datasets/pool_raw", replace


/*====================================================================
======================================================================
							Cleaning Data					
======================================================================
====================================================================*/

use  "${dir}\Datasets/pool_raw", clear

replace country="ukraine" if country=="ukraine_sm"

// Experiment variables


foreach var of varlist c_take_vacc t?_take_vacc  {
	replace `var'="Unsure" if `var'=="Unsure, need more info before I can make up my mind" | `var'=="I am not sure yet"
}

label define l_treatment 0 "No" 1 "Unsure"  2 "Yes"
global treatment c_take_vacc t1_take_vacc t2_take_vacc t3_take_vacc t4_take_vacc t5_take_vacc t6_take_vacc t7_take_vacc t8_take_vacc 


global lab treatment

* Encode with the specific label

foreach label in $lab {
	foreach var in ${`label'} {
	di "`label' - `var'"
	encode `var', gen(l_`var') label(l_`label')
	drop `var'
	rename l_`var' `var'

	}

}


clonevar pool_take_vacc=c_take_vacc
label var pool_take_vacc "COVID-19 vaccine intention"

foreach var of varlist c_take_vacc t?_take_vacc {
	*Pool
	replace pool_take_vacc=`var' if `var'!=.
	
}


foreach var of varlist pool_take_vacc {
	
	* No
	gen no_`var'=0 if `var'!=. 
	replace no_`var'=1 if `var'==0
	label var no_`var' "No - `: variable label `var''"
	* Yes
	gen yes_`var'=0 if `var'!=. 
	replace yes_`var'=1 if `var'==2
	label var yes_`var' "Yes - `: variable label `var''"
	* Unsure 
	gen unsure_`var'=0 if `var'!=. 
	replace unsure_`var'=1 if `var'==1
	label var unsure_`var' "Unsure - `: variable label `var''"
	
}

clonevar unsure_no_pool_take_vacc=unsure_pool_take_vacc
replace unsure_no_pool_take_vacc=. if pool_take_vacc==2

* Generate treatment variables

forval i=0/8 {
	
	gen t`i'=.  
	
}
 
replace t8=0 if c_take_vacc!=. & (country=="DJ" | country=="MK" | country=="XK" | country=="chad" | country=="cmr" | country=="honduras" | country=="iraq_w1" | country=="jordan_w1" | country=="ksa" | country=="kuwait" | country=="lebanon_w1" | country=="libya" | country=="png" | country=="serbia" | country=="tunisia_w1" | country=="wbg")
replace t8=1 if t1_take_vacc!=. 
label var t8 "Experts"
label define t8 0 "Basic Framing" 1 "Experts"
label values t8 t8

replace t7=0 if c_take_vacc!=. & (country=="lebanon_w1" | country=="iraq_w1" | country=="wbg" | country=="libya" | country=="honduras" | country=="tunisia_w1" | country=="cmr")
replace t7=1 if t2_take_vacc!=. 
label var t7 "Experts + Celebrities"
label define t7 0 "Basic Framing" 1 "Experts + Celebrities"
label values t7 t7

replace t6=0 if c_take_vacc!=. & (country=="lebanon_w1" | country=="iraq_w1" | country=="wbg" | country=="libya" | country=="tunisia_w1" | country=="honduras" | country=="cmr" | country=="chad")
replace t6=1 if t3_take_vacc!=. 
label var t6 "Experts + Religious leaders"
label define t6 0 "Basic Framing" 1 "Experts + Religious leaders"
label values t6 t6

replace t5=0 if c_take_vacc!=. & (country=="DJ" | country=="MK" | country=="XK" | country=="congo" | country=="jordan_w1" | country=="jordan_w2" | country=="ksa" | country=="kuwait" | country=="png" | country=="serbia" | country=="tunisia_w2" | country=="gambia")
replace t5=1 if t7_take_vacc!=.
label var t5 "Safety"
label define t5 0 "Basic Framing" 1 "Safety"
label values t5 t5

replace t4=0 if c_take_vacc!=. & (country=="DJ" | country=="MK" | country=="XK" | country=="belize" | country=="haiti" | country=="jamaica" | country=="jordan_w1" | country=="jordan_w2" | country=="ksa" | country=="kuwait" | country=="lebanon_w2" | country=="serbia" | country=="ukraine" | country=="zambia")
replace t4=1 if t5_take_vacc!=.
label var t4 "Efficacy and Pro-Social"
label define t4 0 "Basic Framing" 1 "Efficacy and Pro-Social"
label values t4 t4

replace t3=0 if c_take_vacc!=. & (country=="DJ" | country=="MK" | country=="XK" | country=="congo" | country=="jordan_w1" | country=="jordan_w2" | country=="ksa" | country=="kuwait" | country=="png" | country=="serbia" | country=="tunisia_w2" | country=="ukraine" )
replace t3=1 if t4_take_vacc!=.
label var t3 "Dynamic Social Norms"
label define t3 0 "Basic Framing" 1 "Dynamic Social Norms"
label values t3 t3

replace t2=0 if c_take_vacc!=. & (country=="belize" | country=="haiti" | country=="jamaica" | country=="lebanon_w2" | country=="iraq_w2" | country=="gambia" | country=="tunisia_w3" )
replace t2=1 if t6_take_vacc!=.
label var t2 "Endemic"
label define t2 0 "Basic Framing" 1 "Endemic"
label values t2 t2

replace t1=0 if c_take_vacc!=. & (country=="belize" | country=="haiti" | country=="jamaica" | country=="lebanon_w2" | country=="iraq_w2" | country=="tunisia_w3" )
replace t1=1 if t8_take_vacc!=. 
label var t1 "Variants"
label define t1 0 "Basic Framing" 1 "Variants"
label values t1 t1

replace t0=0 if t1_take_vacc!=. | t2_take_vacc!=. | t3_take_vacc!=. | t4_take_vacc!=. | t5_take_vacc!=. | t6_take_vacc!=. | t7_take_vacc!=. | t8_take_vacc!=. 
replace t0=1 if c_take_vacc!=. 
label var t0 "Basic framing"
label define t0 0 "Treatments" 1 "Basic Framing" 
label values t0 t0


gen treatment_status=.
replace treatment_status=0 if c_take_vacc!=.
replace treatment_status=1 if t1==1
replace treatment_status=2 if t2==1
replace treatment_status=3 if t3==1
replace treatment_status=4 if t4==1
replace treatment_status=5 if t5==1
replace treatment_status=6 if t6==1
replace treatment_status=7 if t7==1
replace treatment_status=8 if t8==1

label define treatment 0 "Basic framing" 8 "Experts" 3 "Dynamic Social Norms" 4 "Efficacy and Pro-Social" 2 "Endemic" 5 "Safety" 1 "Variants" 7 "Experts + Celebrities" 6 "Experts + Religious leaders", replace

label values treatment_status treatment

// Demographic


* Gender
replace gender="Other" if gender!="Male" & gender!="Female" & gender!=""
replace gender="" if gender=="Other"
label define l_gender 0 "Male" 1 "Female" 
global gender gender
label var gender "Female"

* Age
replace age="" if age!="18-29" & age!="30-39" & age!="40-49" & age!="50-59"  & age!="60+" 
label define l_age 1 "18-29" 2 "30-39" 3 "40-49" 4 "50-59" 5 "60+"
global age age 

* gen lastpart = substr(myvar, strpos(myvar, ":") + 1, .)

* Education
replace education="Secondary" if education=="Secondary (high-school)" | education=="Secondary (high school)"
replace education="Tertiary" if education=="Tertiary (Bachelor's degree or higher, any post-secondary)" | education=="Higher education (Bachelor's degree or higher, any post-secondary)" | education=="Higher education (Bachelor's degree or higher)" | education=="Vocational school" 
label define l_education 1 "No education" 2 "Primary" 3 "Secondary" 4 "Tertiary" 
global education education

* Health Work
replace health_work=subinstr(health_work,"Yes, ","",. )
replace health_work="other" if health_work=="other (please specify)"
label define l_health_work 0 "No" 1 "Yes" 2 "doctor" 3 "nurse" 4 "pharmacist" 5 "other" 6 "community health worker"
global health_work health_work

* Health work (dummy)
gen d_health_work=""
replace d_health_work="Other" if health_work=="No"
replace d_health_work="Health work" if health_work!="No" & health_work!=""
label define l_d_health_work 0 "Other" 1 "Health work" 
global d_health_work d_health_work
label variable d_health_work "Health work"

* Education work
replace education_work="Other" if education_work=="No"
replace education_work="Education work" if education_work=="Yes"
label define l_education_work 0 "Other" 1 "Education work" 
global education_work education_work

* Vaccinated against diseases
gen d_vaccinated=""
replace d_vaccinated="Never vaccinated" if vaccinated=="No"
replace d_vaccinated="Previously vaccinated" if vaccinated=="Yes"
label define l_d_vaccinated 0 "Never vaccinated" 1 "Previously vaccinated" 
global d_vaccinated d_vaccinated
label variable d_vaccinated "Previously vaccinated"


global lab gender age education health_work d_health_work education_work d_vaccinated

* Encode with the specific label

foreach label in $lab {
	foreach var in ${`label'} {
	di "`label' - `var'"
	encode `var', gen(l_`var') label(l_`label')
	drop `var'
	rename l_`var' `var'

	}

}

clonevar s_country=country
replace s_country=proper(s_country)
replace s_country=subinstr(s_country,"_W1"," R1",1)
replace s_country=subinstr(s_country,"_W2"," R2",1)
replace s_country=subinstr(s_country,"_W3"," R3",1)

replace s_country="Djibouti" if s_country=="Dj"
replace s_country="Cameroon" if s_country=="Cmr"
replace s_country="Saudi Arabia" if s_country=="Ksa"
replace s_country="North Macedonia" if s_country=="Mk"
replace s_country="Papua New Guinea" if s_country=="Png"
replace s_country="West Bank and Gaza" if s_country=="Wbg"
replace s_country="Kosovo" if s_country=="Xk"

label define vacc_r 28 "Jamaica" 27 "Belize" 26 "Haiti" 25 "Honduras" 24 "Ukraine"   23 "North Macedonia" 22 "Serbia" 21 "Kosovo" 20 "Papua New Guinea" 19 "Gambia"  18 "Cameroon" 17 "Chad"  16 "Zambia"  15 "Congo" 14 "Lebanon R2" 13 "Iraq R2" 12 "Tunisia R3" 11 "Jordan R2" 10 "Tunisia R2" 9 "Djibouti" 8 "Saudi Arabia" 7 "Jordan R1" 6 "Kuwait" 5 "Libya" 4 "Tunisia R1" 3 "West Bank and Gaza" 2 "Iraq R1" 1 "Lebanon R1", replace


label define treat 0 "Basic framing" 1 "Experts" 2 "Dynamic Social Norms" 3 "Efficacy and Pro-Social" 4 "Endemic" 5 "Safety" 6 "Variants" 7 "Experts + Celebrities" 8 "Experts + Religious leaders", replace

encode s_country, gen(Study) label(vacc_r)
label var s_country "Survey"


gen t=.
forval i= 1 / 8 {
	replace t=0 if t`i'==0 & t==.
	replace t=1 if  t`i'==1 & t==.
}

// Moderators

preserve
import excel "${dir}/Datasets/summary_country.xlsx", sheet(Summary_stata) firstrow clear
replace s_country=rtrim(s_country)
rename CountrysCOVID19vaccinationr vacc_rate
rename GDPpercapitaPPPconstant20 gdp_pc
rename COVID19vaccinationrateinsam sample_vacc_rate
rename Startdateofexperiment StartDate
rename Trustdoctorsandnurses14 trust
rename Perceivedvaccinesafety15 vax_safety
rename Perceivedvaccineeffectiveness vax_effectiveness
save "${dir}/Datasets/moderator", replace
restore

merge  m:1 s_country using "${dir}/Datasets/moderator"
drop _merge

drop if t==. | yes_pool_take_vacc==.

foreach var of varlist gender age education d_health_work {
	gen miss_`var'=1 if `var'==.
	replace miss_`var'=0 if `var'!=.
}


/*====================================================================
							Appendix E
====================================================================*/

* Test MCAR MAR
gen mcar_study=""
gen mcar_x2=.
gen mcar_pval=.


local l=1
forval i=1/28 {
	replace mcar_study="`:  label (Study) `i''" in `l'
	capture mcartest gender age education d_health_work if Study==`i', emoutput nolog
	if _rc==0 {
	 scalar chi2_val = r(chi2)
     scalar p_val = r(p)
     replace mcar_x2 = chi2_val in `l'
     replace mcar_pval = p_val in `l'
	}	
	local l = `l' + 1
}

export excel mcar_* using "${dir}/Results/Appendix E", sheetreplace firstrow(variable)


mi set wide
mi register imputed gender age education d_health_work
mi xtset, clear 
* Data set randomized for reproducibility
rsort , seed(1984) id(country id)
version 18, user: mi impute ologit gender yes_pool_take_vacc t  , add(18) rseed(1984)
version 18, user: mi impute ologit age yes_pool_take_vacc t  , replace rseed(1984)
version 18, user: mi impute ologit age yes_pool_take_vacc t  , replace rseed(1984)
version 18, user: mi impute ologit education yes_pool_take_vacc t  ,  replace rseed(1984)
version 18, user: mi impute ologit d_health_work yes_pool_take_vacc t  , replace rseed(1984)
mi unset

gen unique=_n

foreach gr in  "gender_" "age_" "education_" "d_health_work_" {
	forval i=1/10 {
		
	rename `gr'`i'_ `gr'`i'
	}
}

reshape long gender_ age_ education_ d_health_work_ , i(unique) j(obs)

foreach var of varlist gender_ age_ education_ d_health_work_ {
egen i_`var' = mode(`var'), by(unique) minmode
}

reshape wide


foreach gr in "gender" "age" "education" "d_health_work" {
	
	rename i_`gr'_ i_`gr'
	
}

recode i_age (1=0) (2=0) (3=1) (4=1) (5=1) , gen(i_old)
recode i_education (4=0) (2=1) (3=1) (4=1) , gen(i_secondary)

save  "${dir}/Datasets/pool_data", replace

/*====================================================================
======================================================================
							Paper Results					
======================================================================
====================================================================*/

/*====================================================================
								Figure 1
====================================================================*/

use  "${dir}/Datasets/pool_raw", clear

clonevar pool_take_vacc=c_take_vacc
foreach var of varlist c_take_vacc t?_take_vacc {
	replace pool_take_vacc=`var' if `var'!=""
}

* Opt-in 
count // 229,886

* Vacc Status:
count if vacc_covid!="" | pool_take_vacc!="" // 220,599

* Vaccinated users
count if vacc_covid!="" & vacc_covid!="No" // 93,554

* Users who received the experiment (based on eligibility)
count if pool_take_vacc!="" // 123,720

* Contol group
count if c_take_vacc!="" // 62,046

* Treatment group
count if pool_take_vacc!="" &  c_take_vacc=="" // 61,674

/*====================================================================
								Table 1
====================================================================*/

use  "${dir}/Datasets/pool_raw", clear

clonevar pool_take_vacc=c_take_vacc

foreach var of varlist c_take_vacc t?_take_vacc {
	replace pool_take_vacc=`var' if `var'!=""
}

foreach var of varlist gender age education health_work {
	di "all"
	tab `var' if pool_take_vacc!="" | ( vacc_covid!="" & vacc_covid!="No") , miss
	di "experiment"
	tab `var' if pool_take_vacc!="" , miss
	di "vaccinated"
	tab `var' if vacc_covid!="" & vacc_covid!="No" , miss
}

/*====================================================================
								Table 2
====================================================================*/

use "${dir}/Datasets/moderator", clear

gen region=""
replace region="Middle East & North Africa" if s_country=="Lebanon R1" | s_country=="Iraq R1" | s_country=="West Bank and Gaza" | s_country=="Tunisia R1" | s_country=="Libya" | s_country=="Kuwait" | s_country=="Jordan R1" | s_country=="Saudi Arabia" | s_country=="Djibouti" | s_country=="Tunisia R2" | s_country=="Jordan R2" | s_country=="Tunisia R3" | s_country=="Iraq R2" | s_country=="Lebanon R2" 

replace region="Africa" if s_country=="Chad" | s_country=="Cameroon" | s_country=="Gambia" | s_country=="Congo" | s_country=="Zambia"

replace region="Europe, Asia & Pacific" if s_country=="Kosovo" | s_country=="Serbia" | s_country=="North Macedonia" | s_country=="Ukraine"  | s_country=="Papua New Guinea" 

replace region="Latin America & Caribbean" if s_country=="Honduras" | s_country=="Haiti" | s_country=="Belize" | s_country=="Jamaica" 

collapse (count) count=sample_vacc_rate (mean) sample_vacc_rate vacc_rate StartDate gdp_pc trust vax_safety vax_effectiveness ///
(min) min_vacc=sample_vacc_rate min_vacc_rate=vacc_rate min_StartDate=StartDate min_gdp_pc=gdp_pc min_trust=trust min_vax_safety=vax_safety min_vax_effectiveness=vax_effectiveness  ///
(max) max_vacc=sample_vacc_rate max_vacc_rate=vacc_rate max_StartDate=StartDate max_gdp_pc=gdp_pc max_trust=trust max_vax_safety=vax_safety max_vax_effectiveness=vax_effectiveness  ///
,by(region)

export excel using "${dir}/Results/Table 2", sheetreplace  firstrow(variable)

/*====================================================================
							Figure 2 & 3
====================================================================*/

use  "${dir}/Datasets/pool_data", clear

ipdmetan, study(Study) poolvar(t) xlabel(-1(1)1) npts or log title("Overall", size(med)) saving("${dir}/Datasets/F2 T.dta", replace) : logistic yes_pool_take_vacc t i_gender i_age i_education i_d_health_work miss_* , r 


forval i= 8 (-1) 1 {
 
ipdmetan, study(Study) poolvar(t`i') xlabel(-1(1)1) npts or log title("`: label treatment `i''", size(med)) saving("${dir}/Datasets/F2 T`i'.dta", replace) : logistic yes_pool_take_vacc t`i' i_gender i_age i_education i_d_health_work miss_* , r 
graph export "${dir}/Results/Graphs/Figure 2&3 T`i'.png", replace


}

/*====================================================================
								Table 3
====================================================================*/

use "${dir}/Datasets/moderator", clear

drop if s_country=="Iraq R2" | s_country=="Jordan R2" | s_country=="Lebanon R2" | s_country=="Tunisia R2" | s_country=="Tunisia R3" 
replace gdp_pc=gdp_pc/1000
gen month=mofd(StartDate)

sum gdp_pc, d
sum month
sum trust vax_safety vax_effectiveness, d



forval i= 8 (-1) 1 {
	
use "${dir}/Datasets/F2 T`i'.dta", clear


drop if _USE!=1

meta set _ES _LCI _UCI, studysize(_NN) civarlevel(95) studylabel(_LABELS) l(95) eslabel(Std. Diff.) 
rename _LABELS s_country

merge m:1 s_country using "${dir}/Datasets/moderator", nogen

replace gdp_pc=gdp_pc/1000
gen month=mofd(StartDate)
replace sample_vacc_rate=sample_vacc_rate*100


replace gdp_pc=gdp_pc-9.551165
replace month= month-732
replace trust = trust - 3.04578
replace vax_safety = vax_safety - 4.355576
replace vax_effectiveness = vax_effectiveness - 4.429639

if `i'==1 global comb vax_effectiveness
if `i'==1 global comb2 vax_effectiveness
if `i'==2 global comb sample_vacc_rate month vax_effectiveness
if `i'==2 global comb2 sample_vacc_rate month 
if `i'==3 global comb sample_vacc_rate month vax_safety
if `i'==3 global comb2 sample_vacc_rate month 
if `i'==4 global comb sample_vacc_rate month
if `i'==4 global comb2 sample_vacc_rate month
if `i'==5 global comb trust vax_safety
if `i'==5 global comb2 trust vax_safety 
if `i'==6 global comb month 
if `i'==6 global comb2 month trust
if `i'==7 global comb month
if `i'==7 global comb2 month
if `i'==8 global comb month 
if `i'==8 global comb2 month trust

meta regress ${comb} , random nocons

outreg2 using "${dir}/Results/Table 3.xls", dec(3) label append addtext("I2", `e(I2_res)') ctitle("T`i'")

meta regress ${comb2} , random nocons

outreg2 using "${dir}/Results/Table 3.xls", dec(3) label append addtext("I2", `e(I2_res)') ctitle("T`i'")

}

/*====================================================================
				Figure 4, Table 4 & Appendix H
====================================================================*/


use  "${dir}/Datasets/pool_data", clear

forval i= 1 / 8 {
ipdmetan, study(Study) poolvar(1.t`i'#1.i_gender) npts log  or  saving("${dir}/Datasets/t`i'_i_gender.dta", replace) : logistic yes_pool_take_vacc i.t`i'##ib0.i_gender i.t`i'##ib0.i_old i.t`i'##ib0.i_secondary i.t`i'##ib0.i_d_health_work miss_* , r 
}

forval i= 1 / 8 {
ipdmetan, study(Study) poolvar(1.t`i'#1.i_old) npts log or  saving("${dir}/Datasets/t`i'_i_age.dta", replace) : logistic yes_pool_take_vacc i.t`i'##ib0.i_gender i.t`i'##ib0.i_old i.t`i'##ib0.i_secondary i.t`i'##ib0.i_d_health_work miss_* , r 
}

forval i= 1 / 8 {
ipdmetan, study(Study) poolvar(1.t`i'#1.i_secondary)  npts log or  saving("${dir}/Datasets/t`i'_educ.dta", replace)  : logistic yes_pool_take_vacc i.t`i'##ib0.i_gender i.t`i'##ib0.i_old i.t`i'##ib0.i_secondary i.t`i'##ib0.i_d_health_work miss_* , r 
}

forval i= 1 / 8 {
ipdmetan, study(Study) poolvar(1.t`i'#1.i_d_health_work) log npts or  saving("${dir}/Datasets/t`i'_hw.dta", replace)  : logistic yes_pool_take_vacc i.t`i'##ib0.i_gender i.t`i'##ib0.i_old i.t`i'##ib0.i_secondary i.t`i'##ib0.i_d_health_work miss_* , r 
}


foreach j in 1 5 {

foreach gr in "i_gender" "i_age" "educ" "hw" {

use "${dir}/Datasets/t1_`gr'.dta", clear
drop if _USE!=`j'
gen tr=1 


forval i=2/8 {
preserve
use "${dir}/Datasets/t`i'_`gr'.dta", clear
drop if _USE!=`j'
gen tr=`i'
tempfile t`i'
save `t`i''
restore

append using `t`i''
}

label define treatment 0 "Basic framing" 8 "Experts" 3 "Dynamic Social Norms" 4 "Efficacy and Pro-Social" 2 "Endemic" 5 "Safety" 1 "Variants" 7 "Experts + Celebrities" 6 "Experts + Religious leaders", replace

label values tr treatment
gen _pval=2*ttail(_NN,abs(_ES/_seES))
keep _STUDY _EFFECT _NN _pval tr

foreach name in "_EFFECT" "_NN" "_pval" {
rename `name' `gr'`name'
}

tempfile `gr'
save ``gr''

}

use `i_gender'

foreach gr in "i_age" "educ" "hw" {
merge 1:1 _STUDY tr using ``gr'', nogen
}

export excel using "${dir}/Results/Table 4", sheetreplace sheet("int`j'") firstrow(variable)

}

********************************************8

clear all
set obs 20

gen tr=""
gen _ES=.
gen _LCI=.
gen _UCI=.
gen _seES=.
gen _EFFECT=""
gen _NN=.

forval i=1/8 {
	
replace tr="t`i'" in `i'
preserve
use "${dir}/Datasets/t`i'_i_gender.dta", clear
keep if _USE==5
local es=_ES in 1
local se=_seES in 1
local lb=_LCI in 1
local ub=_UCI in 1
local ef=_EFFECT in 1
local n=_NN-18 in 1
restore
replace _ES=`es' in `i'
replace _LCI=`lb' in `i'
replace _UCI=`ub' in `i'
replace _seES=`se' in `i'
replace _EFFECT="`ef'" in `i'
replace _NN=`n' in `i'
}

gen pval=2*ttail(_NN,abs(_ES/_seES))

br tr _ES _LCI _UCI

use "${dir}/Datasets/t1_i_gender.dta", clear
drop if _USE!=1
gen tr=1 


forval i=2/8 {
preserve
use "${dir}/Datasets/t`i'_i_gender.dta", clear
drop if _USE!=1
gen tr=`i'
tempfile t`i'
save `t`i''
restore

append using `t`i''
}
 
label define treatment 0 "Basic framing" 8 "Experts" 3 "Dynamic Social Norms" 4 "Efficacy and Pro-Social" 2 "Endemic" 5 "Safety" 1 "Variants" 7 "Experts + Celebrities" 6 "Experts + Religious leaders", replace

label values tr treatment
 
 gen stat=""
replace stat="Statistically significant" if (_LCI>0 & _UCI>0) | (_LCI<0 & _UCI<0)  
replace stat="Statistically non-significant" if (_LCI>0 & _UCI<0) | (_LCI<0 & _UCI>0)   

stripplot  _ES , ytitle("") xtitle("log Odds ratio") xlabel(-1.6(0.4)1.6, format(%5.1f)) xline(0, lcolor(red)) over(tr) sep(stat) title(Female (base=Male), size(med)) legend(pos(6) size(small)) stack addplot( ///
scatteri 1.3 0.0813863, msize(med) msymbol(D) mcolor(green) || pci 1.3 -0.0965827 1.3 0.2593554, pstyle(p3)  || ///
scatteri 2.3 -0.0362209, msize(med) msymbol(D) mcolor(green) || pci 2.3 -0.2095886 2.3 0.1371468, pstyle(p3)  || ///
scatteri 3.3 -0.0380354, msize(med) msymbol(D) mcolor(green) || pci 3.3 -0.1708093 3.3 0.0947386, pstyle(p3)  || ///
scatteri 4.3 0.1516381, msize(med) msymbol(D) mcolor(green) || pci 4.3 0.0377868 4.3 0.2654893, pstyle(p3)  || ///
scatteri 5.3 0.1010295, msize(med) msymbol(D) mcolor(green) || pci 5.3 -0.029756 5.3 0.231815, pstyle(p3)  || ///
scatteri 6.3 -0.0164756, msize(med) msymbol(D) mcolor(green) || pci 6.3 -0.1113492 6.3 0.078398, pstyle(p3)  || ///
scatteri 7.3 -0.0741054, msize(med) msymbol(D) mcolor(green) || pci 7.3 -0.1720785 7.3 0.0238678, pstyle(p3)  || ///
scatteri 8.3 -0.0177257, msize(med) msymbol(D) mcolor(green) || pci 8.3 -0.1004784 8.3 0.065027, pstyle(p3)  || ///
)
graph export "${dir}/Results/Graphs/Figure 4 i_gender.png", replace


// i_age(40+) (vs i_age 39-)

clear all
set obs 20

gen tr=""
gen _ES=.
gen _LCI=.
gen _UCI=.
gen _seES=.
gen _EFFECT=""
gen _NN=.

forval i=1/8 {
	
replace tr="t`i'" in `i'
preserve
use "${dir}/Datasets/t`i'_i_age.dta", clear
keep if _USE==5
local es=_ES in 1
local se=_seES in 1
local lb=_LCI in 1
local ub=_UCI in 1
local ef=_EFFECT in 1
local n=_NN-18 in 1
restore
replace _ES=`es' in `i'
replace _LCI=`lb' in `i'
replace _UCI=`ub' in `i'
replace _seES=`se' in `i'
replace _EFFECT="`ef'" in `i'
replace _NN=`n' in `i'
}

gen pval=2*ttail(_NN,abs(_ES/_seES))

br tr _ES _LCI _UCI

use "${dir}/Datasets/t1_i_age.dta", clear
drop if _USE!=1
gen tr=1 


forval i=2/8 {
preserve
use "${dir}/Datasets/t`i'_i_age.dta", clear
drop if _USE!=1
gen tr=`i'
tempfile t`i'
save `t`i''
restore

append using `t`i''
}

label define treatment 0 "Basic framing" 8 "Experts" 3 "Dynamic Social Norms" 4 "Efficacy and Pro-Social" 2 "Endemic" 5 "Safety" 1 "Variants" 7 "Experts + Celebrities" 6 "Experts + Religious leaders", replace

label values tr treatment
 
 gen stat=""
replace stat="Statistically significant" if (_LCI>0 & _UCI>0) | (_LCI<0 & _UCI<0)  
replace stat="Statistically non-significant" if (_LCI>0 & _UCI<0) | (_LCI<0 & _UCI>0)

stripplot  _ES , ytitle("") xtitle("log Odds ratio") xlabel(-1.6(0.4)1.6, format(%5.1f)) xline(0, lcolor(red)) over(tr) sep(stat) title(Age 40+ (base=Age 39-), size(med)) legend(pos(6) size(small)) stack addplot( ///
scatteri 1.3 0.0789364, msize(med) msymbol(D) mcolor(green) || pci 1.3 -0.1652288 1.3 0.3231016, pstyle(p3)  || ///
scatteri 2.3 0.2757387, msize(med) msymbol(D) mcolor(green) || pci 2.3 0.0424339 2.3 0.5090434, pstyle(p3)  || ///
scatteri 3.3 -0.0980686, msize(med) msymbol(D) mcolor(green) || pci 3.3 -0.2396844 3.3 0.0435473, pstyle(p3)  || ///
scatteri 4.3 -0.1094282, msize(med) msymbol(D) mcolor(green) || pci 4.3 -0.245976 4.3 0.0271197, pstyle(p3)  || ///
scatteri 5.3 -0.1530656, msize(med) msymbol(D) mcolor(green) || pci 5.3 -0.2913412 5.3 -0.0147901, pstyle(p3)  || ///
scatteri 6.3 -0.0342104, msize(med) msymbol(D) mcolor(green) || pci 6.3 -0.1335345 6.3 0.0651137, pstyle(p3)  || ///
scatteri 7.3 -0.0370103, msize(med) msymbol(D) mcolor(green) || pci 7.3 -0.1379222 7.3 0.0639016, pstyle(p3)  || ///
scatteri 8.3 -0.090581, msize(med) msymbol(D) mcolor(green) || pci 8.3 -0.1770427 8.3 -0.0041193, pstyle(p3)  || ///
)
graph export "${dir}/Results/Graphs/Figure 4 i_age.png", replace

// i_education i_secondary or below (vs tertiary)


clear all
set obs 20

gen tr=""
gen _ES=.
gen _LCI=.
gen _UCI=.
gen _seES=.
gen _EFFECT=""
gen _NN=.

forval i=1/8 {
	
replace tr="t`i'" in `i'
preserve
use "${dir}/Datasets/t`i'_educ.dta", clear
keep if _USE==5
local es=_ES in 1
local se=_seES in 1
local lb=_LCI in 1
local ub=_UCI in 1
local ef=_EFFECT in 1
local n=_NN-18 in 1
restore
replace _ES=`es' in `i'
replace _LCI=`lb' in `i'
replace _UCI=`ub' in `i'
replace _seES=`se' in `i'
replace _EFFECT="`ef'" in `i'
replace _NN=`n' in `i'
}

gen pval=2*ttail(_NN,abs(_ES/_seES))

br tr _ES _LCI _UCI


use "${dir}/Datasets/t1_educ.dta", clear
drop if _USE!=1
gen tr=1 


forval i=2/8 {
preserve
use "${dir}/Datasets/t`i'_educ.dta", clear
drop if _USE!=1
gen tr=`i'
tempfile t`i'
save `t`i''
restore

append using `t`i''
}

label define treatment 0 "Basic framing" 8 "Experts" 3 "Dynamic Social Norms" 4 "Efficacy and Pro-Social" 2 "Endemic" 5 "Safety" 1 "Variants" 7 "Experts + Celebrities" 6 "Experts + Religious leaders", replace

label values tr treatment
 
 gen stat=""
replace stat="Statistically significant" if (_LCI>0 & _UCI>0) | (_LCI<0 & _UCI<0)  
replace stat="Statistically non-significant" if (_LCI>0 & _UCI<0) | (_LCI<0 & _UCI>0)

stripplot  _ES , ytitle("") xtitle("log Odds ratio") xlabel(-1.6(0.4)1.6, format(%5.1f)) xline(0, lcolor(red)) over(tr) sep(stat) title(Secondary or below (base= Tertiary), size(med)) legend(pos(6) size(small)) stack addplot( ///
scatteri 1.3 0.1348431, msize(med) msymbol(D) mcolor(green) || pci 1.3 -0.0442354 1.3 0.3139215, pstyle(p3)  || ///
scatteri 2.3 -0.0345164, msize(med) msymbol(D) mcolor(green) || pci 2.3 -0.2099196 2.3 0.1408868, pstyle(p3)  || ///
scatteri 3.3 0.1064178, msize(med) msymbol(D) mcolor(green) || pci 3.3 -0.0252486 3.3 0.2380842, pstyle(p3)  || ///
scatteri 4.3 0.0529582, msize(med) msymbol(D) mcolor(green) || pci 4.3 -0.0615266 4.3 0.1674431, pstyle(p3)  || ///
scatteri 5.3 0.0663294, msize(med) msymbol(D) mcolor(green) || pci 5.3 -0.0631378 5.3 0.1957965, pstyle(p3)  || ///
scatteri 6.3 -0.031539, msize(med) msymbol(D) mcolor(green) || pci 6.3 -0.1286806 6.3 0.0656027, pstyle(p3)  || ///
scatteri 7.3 -0.0787313, msize(med) msymbol(D) mcolor(green) || pci 7.3 -0.179415 7.3 0.0219523, pstyle(p3)  || ///
scatteri 8.3 -0.0145572, msize(med) msymbol(D) mcolor(green) || pci 8.3 -0.0982453 8.3 0.069131, pstyle(p3)  || ///
)

graph export "${dir}/Results/Graphs/Figure 4 i_education.png", replace

// Health worker (vs rest)

clear all
set obs 20

gen tr=""
gen _ES=.
gen _LCI=.
gen _UCI=.
gen _seES=.
gen _EFFECT=""
gen _NN=.

forval i=1/8 {
	
replace tr="t`i'" in `i'
preserve
use "${dir}/Datasets/t`i'_hw.dta", clear
keep if _USE==5
local es=_ES in 1
local se=_seES in 1
local lb=_LCI in 1
local ub=_UCI in 1
local ef=_EFFECT in 1
local n=_NN-18 in 1
restore
replace _ES=`es' in `i'
replace _LCI=`lb' in `i'
replace _UCI=`ub' in `i'
replace _seES=`se' in `i'
replace _EFFECT="`ef'" in `i'
replace _NN=`n' in `i'
}

gen pval=2*ttail(_NN,abs(_ES/_seES))

br tr _ES _LCI _UCI

use "${dir}/Datasets/t1_hw.dta", clear
drop if _USE!=1
gen tr=1 


forval i=2/8 {
preserve
use "${dir}/Datasets/t`i'_hw.dta", clear
drop if _USE!=1
gen tr=`i'
tempfile t`i'
save `t`i''
restore

append using `t`i''
}

label define treatment 0 "Basic framing" 8 "Experts" 3 "Dynamic Social Norms" 4 "Efficacy and Pro-Social" 2 "Endemic" 5 "Safety" 1 "Variants" 7 "Experts + Celebrities" 6 "Experts + Religious leaders", replace

label values tr treatment
 
 gen stat=""
replace stat="Statistically significant" if (_LCI>0 & _UCI>0) | (_LCI<0 & _UCI<0)  
replace stat="Statistically non-significant" if (_LCI>0 & _UCI<0) | (_LCI<0 & _UCI>0)

stripplot  _ES , ytitle("") xlabel(-1.6(0.4)1.6, format(%5.1f)) xtitle("log Odds ratio") xline(0, lcolor(red)) over(tr) sep(stat) title(Health worker (base= rest), size(med)) legend(pos(6) size(small)) stack addplot( ///
scatteri 1.3 -0.2529992, msize(med) msymbol(D) mcolor(green) || pci 1.3 -0.5180582 1.3 0.0120598, pstyle(p3)  || ///
scatteri 2.3 -0.1848911, msize(med) msymbol(D) mcolor(green) || pci 2.3 -0.4499428 2.3 0.0801606, pstyle(p3)  || ///
scatteri 3.3 0.0218208, msize(med) msymbol(D) mcolor(green) || pci 3.3 -0.2335022 3.3 0.2771438, pstyle(p3)  || ///
scatteri 4.3 -0.0024957, msize(med) msymbol(D) mcolor(green) || pci 4.3 -0.1999351 4.3 0.1949436, pstyle(p3)  || ///
scatteri 5.3 -0.0826373, msize(med) msymbol(D) mcolor(green) || pci 5.3 -0.3347765 5.3 0.1695019, pstyle(p3)  || ///
scatteri 6.3 -0.0119578, msize(med) msymbol(D) mcolor(green) || pci 6.3 -0.1402396 6.3 0.1163239, pstyle(p3)  || ///
scatteri 7.3 0.0835207, msize(med) msymbol(D) mcolor(green) || pci 7.3 -0.0509042 7.3 0.2179456, pstyle(p3)  || ///
scatteri 8.3 0.0606151, msize(med) msymbol(D) mcolor(green) || pci 8.3 -0.0576364 8.3 0.1788666, pstyle(p3)  || ///
 )

graph export "${dir}/Results/Graphs/Figure 4 health worker.png", replace


/*====================================================================
						Figure 5 & 6
====================================================================*/

use  "${dir}/Datasets/pool_data", clear

foreach i in 8 5 {

ipdmetan, study(Study) poolvar(t`i') xlabel(-1(1)1) npts or log title("Age 39-", size(med)) saving("${dir}/Datasets/ipd t`i' young.dta", replace) : logistic yes_pool_take_vacc t`i' ib0.i_gender ib0.i_secondary ib0.i_d_health_work miss_* if i_old==0 , r 
graph export "${dir}/Results/Graphs/Figure 5_6 young T`i'.png", replace

ipdmetan, study(Study) poolvar(t`i') xlabel(-1(1)1) npts or log title("Age 40+", size(med)) saving("${dir}/Datasets/ipd t`i' old.dta", replace) : logistic yes_pool_take_vacc t`i' ib0.i_gender ib0.i_secondary ib0.i_d_health_work miss_* if i_old==1, r 
graph export "${dir}/Results/Graphs/Figure 5_6 old T`i'.png", replace
}

/*====================================================================
							Figure 7
====================================================================*/

forval i= 4/4 {

ipdmetan, study(Study) poolvar(t`i') xlabel(-1(1)1) npts or log title("Male", size(med))  saving("${dir}/Datasets/ipd t`i' male.dta", replace) : logistic yes_pool_take_vacc t`i' ib0.i_old ib0.i_secondary ib0.i_d_health_work miss_* if i_gender==0 , r 
graph export "${dir}/Results/Graphs/Figure 7 male T`i'.png", replace

ipdmetan, study(Study) poolvar(t`i') xlabel(-1(1)1) npts or log title("Female", size(med))  saving("${dir}/Datasets/ipd t`i' female.dta", replace) : logistic yes_pool_take_vacc t`i' ib0.i_old ib0.i_secondary ib0.i_d_health_work miss_*  if i_gender==1, r 
graph export "${dir}/Results/Graphs/Figure 7 female T`i'.png", replace
}



/*====================================================================
							Appendix A 
====================================================================*/

use "${dir}/Datasets/moderator", clear

/*====================================================================
							Appendix B 
====================================================================*/

use  "${dir}/Datasets/pool_data", clear

tab Study treatment_status, row

/*====================================================================
					Appendix D - Table SD1
====================================================================*/


use "${dir}/Datasets/moderator_individual_level", clear


* Intraclass correlation coefficients

foreach var of varlist trust vax_safety vax_effectiveness {
mixed `var' || country:, reml
estat icc
}

foreach var of varlist  trust vax_safety vax_effectiveness{

* Calculate the mean rating for each group
egen m_`var' = mean( `var' ), by(country)

* Calculate the observed variance of ratings within each group
egen v_`var' = mean((`var' - m_`var')^2), by(country)

* Ratings range from 1 to 5
if "`var'"=="Q11E" local A = 4
if "`var'"!="Q11E" local A = 5
local S2E = ((`A'^2 - 1) / 12)

* Create a variable for the expected variance
gen ev_`var' = `S2E'

* Calculate rwg for each group
gen rwg_`var' = 1 - (v_`var' / ev_`var')

}

* By country
collapse  (mean)rwg_*, by(country)

export excel using "$dir/Results/TableSD1.xlsx", replace 

* Total
collapse  (mean)rwg_*

/*====================================================================
							Appendix F
====================================================================*/


use  "${dir}/Datasets/pool_data", clear

forval i= 8 (-1) 1 {

ipdmetan, study(Study) poolvar(t`i') npts or title("`: label treatment `i''", size(med)) saving("${dir}/Datasets/ApE T`i'.dta", replace) : logistic yes_pool_take_vacc t`i' i_gender i_age i_education i_d_health_work miss_*, r 
graph export "${dir}/Results/Graphs/ApF T`i'.png", replace


}

ipdmetan, study(Study) poolvar(t) npts or title("Combined", size(med)) saving("${dir}/Datasets/ApE T.dta", replace) : logistic yes_pool_take_vacc t`i' i_gender i_age i_education i_d_health_work miss_*, r 
graph export "${dir}/Results/Graphs/ApF T.png", replace


// 1 (all) 5 (comb)

foreach j in 1 5 {
	
forval i=1/8 {
use "${dir}/Datasets/ApE T`i'.dta", clear
drop if _USE!=`j'
gen tr=`i'
tempfile t`i'
save `t`i''
}

use `t1'
forval i=2/8 {
append  using `t`i'', 
}

label define treatment 0 "Basic framing" 8 "Experts" 3 "Dynamic Social Norms" 4 "Efficacy and Pro-Social" 2 "Endemic" 5 "Safety" 1 "Variants" 7 "Experts + Celebrities" 6 "Experts + Religious leaders", replace

label values tr treatment
gen _pval=2*ttail(_NN,abs(_ES/_seES))
keep _STUDY _EFFECT _NN _pval tr


export excel using "${dir}/Results/Appendix F", sheetreplace sheet("int`j'") firstrow(variable)

}


/*====================================================================
							Appendix G
====================================================================*/

use "${dir}/Datasets/moderator", clear

drop if s_country=="Iraq R2" | s_country=="Jordan R2" | s_country=="Lebanon R2" | s_country=="Tunisia R2" | s_country=="Tunisia R3" 
replace gdp_pc=gdp_pc/1000
gen month=mofd(StartDate)

sum gdp_pc, d
sum month
sum trust vax_safety vax_effectiveness, d



forval i= 8 (-1) 1 {
	
use "${dir}/Datasets/F2 T`i'.dta", clear


drop if _USE!=1

meta set _ES _LCI _UCI, studysize(_NN) civarlevel(95) studylabel(_LABELS) l(95) eslabel(Std. Diff.) 
rename _LABELS s_country

merge m:1 s_country using "${dir}/Datasets/moderator", nogen

replace gdp_pc=gdp_pc/1000
gen month=mofd(StartDate)
replace sample_vacc_rate=sample_vacc_rate*100


replace gdp_pc=gdp_pc-9.551165
replace month= month-732
replace trust = trust - 3.04578
replace vax_safety = vax_safety - 4.355576
replace vax_effectiveness = vax_effectiveness - 4.429639

foreach var of varlist sample_vacc_rate month gdp_pc trust  vax_* {
	capture meta regress `var', random nocons
	if _rc==0 {
	meta regress `var', random nocons 
	outreg2 using "${dir}/Results/Appendix G.xls", dec(3) label append addtext("I2", `e(I2_res)') ctitle("T`i'")
	}

}
}

/*====================================================================
							Appendix H
====================================================================*/

* From Table 4